@@ -3,7 +3,8 @@
  * @ingroup MAT
  */
 /*
- * Copyright (c) 2005-2021, Christopher C. Hulbert
+ * Copyright (c) 2015-2022, The matio contributors
+ * Copyright (c) 2005-2014, Christopher C. Hulbert
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,6 +32,7 @@
 #include "matio_private.h"
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 
 #if HAVE_ZLIB
 
@@ -125,8 +127,8 @@ InflateSkip(mat_t *mat, z_streamp z, int nBytes, size_t *bytesread)
     }
 
     if ( z->avail_in ) {
-        const long offset = -(long)z->avail_in;
-        (void)fseek((FILE *)mat->fp, offset, SEEK_CUR);
+        const mat_off_t offset = -(mat_off_t)z->avail_in;
+        (void)fseeko((FILE *)mat->fp, offset, SEEK_CUR);
         if ( NULL != bytesread ) {
             *bytesread -= z->avail_in;
         }
@@ -144,7 +146,6 @@ InflateSkip(mat_t *mat, z_streamp z, int nBytes, size_t *bytesread)
  * @param z zlib compression stream
  * @param data_type Data type (matio_types enumerations)
  * @param len Number of elements of datatype @c data_type to skip
- * @param[out] bytesread Number of bytes read from the file
  * @retval 0 on success
 
  */
@@ -185,8 +186,10 @@ int
 InflateRankDims(mat_t *mat, z_streamp z, void *buf, size_t nBytes, mat_uint32_t **dims,
                 size_t *bytesread)
 {
-    mat_int32_t tag[2];
-    int rank, i, err;
+    mat_uint32_t tag[2];
+    mat_uint32_t rank, i;
+    int err;
+    size_t nbytes = 0;
 
     if ( buf == NULL )
         return MATIO_E_BAD_ARGUMENT;
@@ -195,11 +198,11 @@ InflateRankDims(mat_t *mat, z_streamp z, void *buf, size_t nBytes, mat_uint32_t
     if ( err ) {
         return err;
     }
-    tag[0] = *(int *)buf;
-    tag[1] = *((int *)buf + 1);
+    tag[0] = *(mat_uint32_t *)buf;
+    tag[1] = *((mat_uint32_t *)buf + 1);
     if ( mat->byteswap ) {
-        Mat_int32Swap(tag);
-        Mat_int32Swap(tag + 1);
+        Mat_uint32Swap(tag);
+        Mat_uint32Swap(tag + 1);
     }
     if ( (tag[0] & 0x0000ffff) != MAT_T_INT32 ) {
         Mat_Critical("InflateRankDims: Reading dimensions expected type MAT_T_INT32");
@@ -210,17 +213,28 @@ InflateRankDims(mat_t *mat, z_streamp z, void *buf, size_t nBytes, mat_uint32_t
         i = 8 - (rank % 8);
     else
         i = 0;
+
+    if ( rank > INT_MAX - i - 2 ) {
+        Mat_Critical("InflateRankDims: Reading dimensions expected rank in integer range");
+        return MATIO_E_FILE_FORMAT_VIOLATION;
+    }
     rank += i;
 
-    if ( sizeof(mat_uint32_t) * (rank + 2) <= nBytes ) {
-        err = Inflate(mat, z, (mat_int32_t *)buf + 2, (unsigned int)rank, bytesread);
+    err = Mul(&nbytes, rank + 2, sizeof(mat_uint32_t));
+    if ( err ) {
+        Mat_Critical("Integer multiplication overflow");
+        return err;
+    }
+
+    if ( nbytes <= nBytes ) {
+        err = Inflate(mat, z, (mat_uint32_t *)buf + 2, rank, bytesread);
     } else {
         /* Cannot use too small buf, but can allocate output buffer dims */
         *dims = (mat_uint32_t *)calloc(rank, sizeof(mat_uint32_t));
         if ( NULL != *dims ) {
-            err = Inflate(mat, z, *dims, (unsigned int)rank, bytesread);
+            err = Inflate(mat, z, *dims, rank, bytesread);
         } else {
-            *((mat_int32_t *)buf + 1) = 0;
+            *((mat_uint32_t *)buf + 1) = 0;
             Mat_Critical("Error allocating memory for dims");
             return MATIO_E_OUT_OF_MEMORY;
         }
@@ -292,8 +306,8 @@ Inflate(mat_t *mat, z_streamp z, void *buf, unsigned int nBytes, size_t *bytesre
     }
 
     if ( z->avail_in ) {
-        const long offset = -(long)z->avail_in;
-        (void)fseek((FILE *)mat->fp, offset, SEEK_CUR);
+        const mat_off_t offset = -(mat_off_t)z->avail_in;
+        (void)fseeko((FILE *)mat->fp, offset, SEEK_CUR);
         if ( NULL != bytesread ) {
             *bytesread -= z->avail_in;
         }
@@ -319,7 +333,6 @@ Inflate(mat_t *mat, z_streamp z, void *buf, unsigned int nBytes, size_t *bytesre
  * @param z zlib compression stream
  * @param buf Pointer to store the uncompressed data
  * @param nBytes Number of uncompressed bytes to inflate
- * @param[out] bytesread Number of bytes read from the file
  * @retval 0 on success
 
  */
@@ -389,8 +402,8 @@ InflateData(mat_t *mat, z_streamp z, void *buf, unsigned int nBytes)
     }
 
     if ( z->avail_in ) {
-        const long offset = -(long)z->avail_in;
-        (void)fseek((FILE *)mat->fp, offset, SEEK_CUR);
+        const mat_off_t offset = -(mat_off_t)z->avail_in;
+        (void)fseeko((FILE *)mat->fp, offset, SEEK_CUR);
         bytesread -= z->avail_in;
         z->avail_in = 0;
     }
