@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2005-2021, Christopher C. Hulbert
+ * Copyright (c) 2015-2022, The matio contributors
+ * Copyright (c) 2005-2014, Christopher C. Hulbert
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -133,7 +134,7 @@ slab_get_select(char *open, char *close, int rank, int *start, int *stride, int
                 if ( !strcmp(valptr, "end") ) {
                     edge[dim] = -1;
                 } else {
-                    i = atoi(valptr);
+                    i = (int)strtol(valptr, NULL, 10);
                     edge[dim] = i;
                 }
             } else if ( nvals == 1 ) {
@@ -141,7 +142,7 @@ slab_get_select(char *open, char *close, int rank, int *start, int *stride, int
                 if ( !strcmp(valptr, "end") ) {
                     edge[dim] = -1;
                 } else {
-                    i = atoi(valptr);
+                    i = (int)strtol(valptr, NULL, 10);
                     edge[dim] = i;
                 }
             } else if ( nvals == 0 ) {
@@ -150,7 +151,7 @@ slab_get_select(char *open, char *close, int rank, int *start, int *stride, int
                     start[dim] = -1;
                     edge[dim] = -1;
                 } else {
-                    i = atoi(valptr);
+                    i = (int)strtol(valptr, NULL, 10);
                     start[dim] = i - 1;
                     edge[dim] = i;
                 }
@@ -170,7 +171,7 @@ slab_get_select(char *open, char *close, int rank, int *start, int *stride, int
                 else
                     fprintf(stderr, "Too many inputs to dim %d", dim + 1);
             } else {
-                i = atoi(valptr);
+                i = (int)strtol(valptr, NULL, 10);
                 if ( nvals == 0 )
                     start[dim] = i - 1;
                 else if ( nvals == 1 )
@@ -195,7 +196,7 @@ slab_get_select(char *open, char *close, int rank, int *start, int *stride, int
                 else
                     fprintf(stderr, "Too many inputs to dim %d", dim + 1);
             } else {
-                i = atoi(valptr);
+                i = (int)strtol(valptr, NULL, 10);
                 if ( nvals == 0 ) {
                     start[dim] = i - 1;
                     edge[dim] = i;
@@ -307,7 +308,7 @@ read_selected_data(mat_t *mat, matvar_t **_matvar, char *index_str)
 {
     char *next_tok_pos, next_tok = 0;
     char *open = NULL, *close = NULL;
-    int err, i = 0, j, done = 0;
+    int err = 1, i = 0, j, done = 0;
     matvar_t *matvar = *_matvar;
 
     next_tok_pos = get_next_token(index_str);
@@ -558,8 +559,6 @@ print_whos(matvar_t *matvar)
         printf("  %10" SIZE_T_FMTSTR, nbytes);
     }
     printf("  %-18s\n", mxclass[matvar->class_type]);
-
-    return;
 }
 
 static size_t indent = 0;
@@ -861,11 +860,12 @@ main(int argc, char *argv[])
                 break;
             case 'H':
                 Mat_Help(helpstr);
-                exit(EXIT_SUCCESS);
+                /* Note: Mat_Help() calls exit() */
             case 'V':
                 printf(
                     "%s %s\nWritten by Christopher Hulbert\n\n"
-                    "Copyright(C) 2006-2021, Christopher C. Hulbert\n",
+                    "Copyright(C) 2015-2022, The matio contributors\n"
+                    "Copyright(C) 2006-2014, Christopher C. Hulbert\n",
                     prog_name, MATIO_VERSION_STR);
                 exit(EXIT_SUCCESS);
             default:
